@@ -309,7 +309,7 @@ module Extensions
     alias parse_content_as content_model
 
     def positional_attributes *value
-      option :positional_attrs, value.flatten
+      option :positional_attrs, (value.flatten.map {|name| name.to_s })
     end
     alias name_positional_attributes positional_attributes
     # NOTE positional_attrs alias is deprecated
@@ -485,7 +485,7 @@ module Extensions
   # to add content to the header.
   class DocinfoProcessor < Processor
     def initialize config = {}
-      super config
+      super
       @config[:location] ||= :head
     end
 
@@ -1109,6 +1109,9 @@ module Extensions
     # Public: Checks whether any {BlockProcessor} extensions are registered to
     # handle the specified block name appearing on the specified context.
     #
+    # This method assumes you've called blocks? first to check whether any
+    # block extensions are registered.
+    #
     # Returns the [Extension] proxy object for the BlockProcessor that matches
     # the block name and context or false if no match is found.
     def registered_for_block? name, context
@@ -1124,6 +1127,9 @@ module Extensions
     #
     # name - the String or Symbol (coersed to a Symbol) macro name
     #
+    # This method assumes you've called blocks? first to check whether any
+    # block extensions are registered.
+    #
     # Returns the [Extension] object stored in the registry that proxies the
     # corresponding BlockProcessor or nil if a match is not found.
     def find_block_extension name
@@ -1200,6 +1206,9 @@ module Extensions
     #
     # name - the String or Symbol (coersed to a Symbol) macro name
     #
+    # This method assumes you've called block_macros? first to check whether any
+    # block macro extensions are registered.
+    #
     # Returns the [Extension] proxy object for the BlockMacroProcessor that matches
     # the macro name or false if no match is found.
     #--
@@ -1213,6 +1222,9 @@ module Extensions
     #
     # name - the String or Symbol (coersed to a Symbol) macro name
     #
+    # This method assumes you've called block_macros? first to check whether any
+    # block macro extensions are registered.
+    #
     # Returns the [Extension] object stored in the registry that proxies the
     # corresponding BlockMacroProcessor or nil if a match is not found.
     def find_block_macro_extension name
@@ -1289,6 +1301,9 @@ module Extensions
     #
     # name - the String or Symbol (coersed to a Symbol) macro name
     #
+    # This method assumes you've called inline_macros? first to check whether any
+    # inline macro extensions are registered.
+    #
     # Returns the [Extension] proxy object for the InlineMacroProcessor that matches
     # the macro name or false if no match is found.
     def registered_for_inline_macro? name
@@ -1300,6 +1315,9 @@ module Extensions
     #
     # name - the String or Symbol (coersed to a Symbol) macro name
     #
+    # This method assumes you've called inline_macros? first to check whether any
+    # inline macro extensions are registered.
+    #
     # Returns the [Extension] object stored in the registry that proxies the
     # corresponding InlineMacroProcessor or nil if a match is not found.
     def find_inline_macro_extension name
@@ -1309,6 +1327,9 @@ module Extensions
     # Public: Retrieves the {Extension} proxy objects for all
     # InlineMacroProcessor instances in this registry.
     #
+    # This method assumes you've called inline_macros? first to check whether any
+    # inline macro extensions are registered.
+    #
     # Returns an [Array] of Extension proxy objects.
     def inline_macros
       @inline_macro_extensions.values
@@ -1338,7 +1359,7 @@ module Extensions
 
     def add_document_processor kind, args, &block
       kind_name = kind.to_s.tr '_', ' '
-      kind_class_symbol = kind_name.split.map {|it| it.capitalize }.join.to_sym
+      kind_class_symbol = kind_name.split.map {|segment| segment.capitalize }.join.to_sym
       kind_class = Extensions.const_get kind_class_symbol, false
       kind_java_class = (defined? ::AsciidoctorJ) ? (::AsciidoctorJ::Extensions.const_get kind_class_symbol, false) : nil
       kind_store = instance_variable_get(%(@#{kind}_extensions).to_sym) || instance_variable_set(%(@#{kind}_extensions).to_sym, [])
@@ -1382,7 +1403,7 @@ module Extensions
 
     def add_syntax_processor kind, args, &block
       kind_name = kind.to_s.tr '_', ' '
-      kind_class_symbol = (kind_name.split.map {|it| it.capitalize } << 'Processor').join.to_sym
+      kind_class_symbol = (kind_name.split.map {|segment| segment.capitalize } << 'Processor').join.to_sym
       kind_class = Extensions.const_get kind_class_symbol, false
       kind_java_class = (defined? ::AsciidoctorJ) ? (::AsciidoctorJ::Extensions.const_get kind_class_symbol, false) : nil
       kind_store = instance_variable_get(%(@#{kind}_extensions).to_sym) || instance_variable_set(%(@#{kind}_extensions).to_sym, {})
