@@ -1368,17 +1368,17 @@ context 'Extensions' do
         ++++
         EOS
         expected = <<~'EOS'.chop
-        target="", attributes={}
-        target="value,key=val", attributes={1=>"value", "key"=>"val", "name"=>"value"}
-        target="", attributes={"text"=>""}
-        target="[text]", attributes={"text"=>"[text]"}
-        target="target", attributes={}
-        target="target", attributes={1=>"value", "key"=>"val", "name"=>"value"}
-        target="target", attributes={"text"=>""}
-        target="target", attributes={"text"=>"[text]"}
-        target="target", attributes={}
+        target="",attributes={}
+        target="value,key=val",attributes={1=>"value","key"=>"val","name"=>"value"}
+        target="",attributes={"text"=>""}
+        target="[text]",attributes={"text"=>"[text]"}
+        target="target",attributes={}
+        target="target",attributes={1=>"value","key"=>"val","name"=>"value"}
+        target="target",attributes={"text"=>""}
+        target="target",attributes={"text"=>"[text]"}
+        target="target",attributes={}
         EOS
-        output = convert_string_to_embedded input
+        output = ((convert_string_to_embedded input).gsub ' => ', '=>').gsub ', ', ','
         assert_equal expected, output
       ensure
         Asciidoctor::Extensions.unregister_all
@@ -1543,6 +1543,25 @@ context 'Extensions' do
       end
     end
 
+    test 'should coerce names of positional attributes to strings' do
+      begin
+        Asciidoctor::Extensions.register do
+          inline_macro :attrs do
+            match_format :short
+            positional_attributes :a, 'b'
+            process do |parent, _, attrs|
+              create_inline_pass parent, %(a=#{attrs['a']},b=#{attrs['b']})
+            end
+          end
+        end
+
+        output = convert_string_to_embedded 'attrs:[A,B]', doctype: :inline
+        assert_equal 'a=A,b=B', output
+      ensure
+        Asciidoctor::Extensions.unregister_all
+      end
+    end
+
     test 'should not carry over attributes if block processor returns nil' do
       begin
         Asciidoctor::Extensions.register do
