@@ -193,6 +193,12 @@ context 'Invoker' do
     assert_match(/WARNING/, warnings)
   end
 
+  test 'should not emit any unexpected warnings' do
+    input_path = fixture_path 'basic.adoc'
+    output = run_command(asciidoctor_cmd, '-o', '/dev/null', '-w', input_path) {|out| out.read }
+    assert_empty output
+  end
+
   test 'should enable script warnings if -w flag is specified' do
     old_verbose, $VERBOSE = $VERBOSE, false
     begin
@@ -797,6 +803,24 @@ context 'Invoker' do
     end
   end
 
+  test 'should ignore SOURCE_DATE_EPOCH is value is empty' do
+    old_source_date_epoch = ENV.delete 'SOURCE_DATE_EPOCH'
+    begin
+      ENV['SOURCE_DATE_EPOCH'] = ''
+      sample_filepath = fixture_path 'sample.adoc'
+      invoker = invoke_cli_to_buffer %w(-o /dev/null), sample_filepath
+      doc = invoker.document
+      current_year = Time.now.strftime '%F'
+      assert (doc.attr 'localyear').to_i >= (current_year.to_i - 1)
+    ensure
+      if old_source_date_epoch
+        ENV['SOURCE_DATE_EPOCH'] = old_source_date_epoch
+      else
+        ENV.delete 'SOURCE_DATE_EPOCH'
+      end
+    end
+  end
+
   test 'should fail if SOURCE_DATE_EPOCH is malformed' do
     old_source_date_epoch = ENV.delete 'SOURCE_DATE_EPOCH'
     begin
